"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
//#region @backend
var child = require("child_process");
var fse = require("fs-extra");
var path = require("path");
var _ = require("lodash");
var incremental_compiler_1 = require("incremental-compiler");
var helpers_1 = require("../helpers");
var BackendCompilation = /** @class */ (function (_super) {
    tslib_1.__extends(BackendCompilation, _super);
    function BackendCompilation(
    /**
     * Output folder
     * Ex. dist
     */
    outFolder, 
    /**
     * Source location
     * Ex. src | components
     */
    location, 
    /**
     * Current cwd same for browser and backend
     * but browser project has own compilation folder
     * Ex. /home/username/project/myproject
     */
    cwd) {
        var _this = _super.call(this, {
            folderPath: [path.join(cwd, location)],
            notifyOnFileUnlink: true,
        }) || this;
        _this.outFolder = outFolder;
        _this.location = location;
        _this.cwd = cwd;
        _this.isEnableCompilation = true;
        _this.compilerName = 'Backend Compiler';
        return _this;
    }
    BackendCompilation_1 = BackendCompilation;
    Object.defineProperty(BackendCompilation.prototype, "compilationFolderPath", {
        get: function () {
            if (_.isString(this.location) && _.isString(this.cwd)) {
                return path.join(this.cwd, this.location);
            }
        },
        enumerable: true,
        configurable: true
    });
    BackendCompilation.prototype.tscCompilation = function (cwd, watch, outDir, generateDeclarations, tsExe, diagnostics) {
        if (watch === void 0) { watch = false; }
        if (generateDeclarations === void 0) { generateDeclarations = false; }
        if (tsExe === void 0) { tsExe = 'tsc'; }
        if (diagnostics === void 0) { diagnostics = false; }
        if (!this.isEnableCompilation) {
            console.log("Compilation disabled for " + _.startCase(BackendCompilation_1.name));
            return;
        }
        var params = [
            watch ? '-w' : '',
            outDir ? "--outDir " + outDir : '',
            !watch ? '--noEmitOnError true' : '',
            diagnostics ? ' --extendedDiagnostics' : '',
            "--preserveWatchOutput"
        ];
        var commandJsAndMaps = tsExe + " -d false  " + params.join(' ');
        var commandDts = tsExe + "  " + params.join(' ');
        // console.log(`(${this.compilerName}) Execute first command : ${commandJsAndMaps}    # inside: ${cwd}`)
        // console.log(`(${this.compilerName}) Execute second command : ${commandDts}    # inside: ${cwd}`)
        if (watch) {
            helpers_1.Helpers.log(child.exec(commandJsAndMaps, { cwd: cwd }));
            if (generateDeclarations) {
                helpers_1.Helpers.log(child.exec(commandDts, { cwd: cwd }));
            }
        }
        else {
            try {
                child.execSync(commandJsAndMaps, {
                    cwd: cwd,
                    stdio: [0, 1, 2]
                });
            }
            catch (e) {
                console.error("Compilation error: " + e);
                process.exit(1);
            }
            if (generateDeclarations) {
                try {
                    child.execSync(commandDts, {
                        cwd: cwd,
                        stdio: [0, 1, 2]
                    });
                }
                catch (e) {
                    console.error("Compilation error: " + e);
                    process.exit(1);
                }
            }
        }
    };
    BackendCompilation.prototype.compile = function (watch) {
        if (watch === void 0) { watch = false; }
        this.tscCompilation(this.compilationFolderPath, watch, "../" + this.outFolder, true);
    };
    BackendCompilation.prototype.syncAction = function (filesPathes) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var outDistPath;
            return tslib_1.__generator(this, function (_a) {
                outDistPath = path.join(this.cwd, this.outFolder);
                // Helpers.System.Operations.tryRemoveDir(outDistPath)
                if (!fse.existsSync(outDistPath)) {
                    fse.mkdirpSync(outDistPath);
                }
                this.compile();
                return [2 /*return*/];
            });
        });
    };
    BackendCompilation.prototype.preAsyncAction = function () {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            return tslib_1.__generator(this, function (_a) {
                this.compile(true);
                return [2 /*return*/];
            });
        });
    };
    Object.defineProperty(BackendCompilation.prototype, "tsConfigName", {
        get: function () {
            return 'tsconfig.json';
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(BackendCompilation.prototype, "tsConfigBrowserName", {
        get: function () {
            return 'tsconfig.browser.json';
        },
        enumerable: true,
        configurable: true
    });
    var BackendCompilation_1;
    BackendCompilation = BackendCompilation_1 = tslib_1.__decorate([
        incremental_compiler_1.IncCompiler.Class({ className: 'BackendCompilation' }),
        tslib_1.__metadata("design:paramtypes", [String, String, String])
    ], BackendCompilation);
    return BackendCompilation;
}(incremental_compiler_1.IncCompiler.Base));
exports.BackendCompilation = BackendCompilation;
//#endregion
//# sourceMappingURL=compilation-backend.js.map