"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var fse = require("fs-extra");
var rimraf = require("rimraf");
var path = require("path");
var _ = require("lodash");
var helpers_1 = require("../helpers");
var browser_code_cut_1 = require("./browser-code-cut");
var compilation_backend_1 = require("./compilation-backend");
var incremental_compiler_1 = require("incremental-compiler");
var BroswerCompilation = /** @class */ (function (_super) {
    tslib_1.__extends(BroswerCompilation, _super);
    function BroswerCompilation(
    /**
     * Relative path for browser temporary src
     * Ex.   tmp-src-dist-browser
     */
    sourceOutBrowser, outFolder, location, cwd, backendOutFolder, customCompiler) {
        var _this = _super.call(this, outFolder, location, cwd) || this;
        _this.sourceOutBrowser = sourceOutBrowser;
        _this.backendOutFolder = backendOutFolder;
        _this.customCompiler = customCompiler;
        _this.compilerName = 'Browser standard compiler';
        return _this;
    }
    Object.defineProperty(BroswerCompilation.prototype, "compilationFolderPath", {
        get: function () {
            if (_.isString(this.sourceOutBrowser) && _.isString(this.cwd)) {
                return path.join(this.cwd, this.sourceOutBrowser);
            }
        },
        enumerable: true,
        configurable: true
    });
    BroswerCompilation.prototype.syncAction = function (files) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var dereference, source, dest;
            return tslib_1.__generator(this, function (_a) {
                // console.log(files)
                if (fse.existsSync(this.compilationFolderPath)) {
                    rimraf.sync(this.compilationFolderPath);
                }
                fse.mkdirpSync(this.compilationFolderPath);
                dereference = true;
                // console.log(`copying ${path.join(this.cwd, this.location)}/ to  ${this.compilationFolderPath} dereference: ${dereference},`)
                // TODO this may be replaced by filesPathes
                helpers_1.Helpers.System.Operations.tryCopyFrom(path.join(this.cwd, this.location) + "/", this.compilationFolderPath, {
                    dereference: dereference,
                    filter: function (src, dest) {
                        return copyToBrowserSrcCodition(src);
                    }
                });
                // console.log('browser', this.filesAndFoldesRelativePathes.slice(0, 5))
                this.initCodeCut(files);
                source = path.join(this.cwd, this.tsConfigBrowserName);
                dest = path.join(this.cwd, this.sourceOutBrowser, this.tsConfigName);
                fse.copyFileSync(source, dest);
                this.codecut.files();
                this.compile();
                return [2 /*return*/];
            });
        });
    };
    BroswerCompilation.prototype.initCodeCut = function (filesPathes) {
        var _this = this;
        filesPathes = filesPathes.map(function (f) {
            return f.replace(path.join(_this.cwd, _this.location), '').replace(/^\//, '');
        });
        this.codecut = new browser_code_cut_1.CodeCut(this.compilationFolderPath, filesPathes, {
            replacements: [
                ["@backendFunc",
                    "return undefined;\n"],
                "@backend"
            ]
        });
    };
    BroswerCompilation.prototype.asyncAction = function (event) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var absoluteFilePath, relativeFilePath, destinationFilePath, orgFil;
            return tslib_1.__generator(this, function (_a) {
                absoluteFilePath = event.fileAbsolutePath;
                relativeFilePath = absoluteFilePath.replace(path.join(this.cwd, this.location), '');
                destinationFilePath = path.join(this.cwd, this.sourceOutBrowser, relativeFilePath);
                // console.log('this.cwd', this.cwd)
                // console.log('this.sourceOutBrowser', this.sourceOutBrowser)
                // console.log('destinationFilePath', destinationFilePath)
                // console.log('[asyncAction][morphi][cb] absoluteFilePath', absoluteFilePath)
                // console.log('[asyncAction][morphi][cb] destinationFilePath', destinationFilePath)
                if (copyToBrowserSrcCodition(absoluteFilePath)) {
                    if (event.eventName === 'unlink') {
                        if (fse.existsSync(destinationFilePath)) {
                            fse.unlinkSync(destinationFilePath);
                        }
                        if (['module', 'component']
                            .map(function (c) { return "." + c + ".ts"; })
                            .filter(function (c) { return destinationFilePath.endsWith(c); }).length > 0) {
                            orgFil = destinationFilePath + ".orginal";
                            if (fse.existsSync(orgFil)) {
                                fse.unlinkSync(orgFil);
                            }
                        }
                    }
                    else {
                        if (fse.existsSync(absoluteFilePath)) {
                            if (!fse.existsSync(path.dirname(destinationFilePath))) {
                                fse.mkdirpSync(path.dirname(destinationFilePath));
                            }
                            if (fse.existsSync(destinationFilePath) && fse.lstatSync(destinationFilePath).isDirectory()) {
                                fse.removeSync(destinationFilePath);
                            }
                            fse.copyFileSync(absoluteFilePath, destinationFilePath);
                        }
                    }
                    this.codecut.file(destinationFilePath);
                }
                return [2 /*return*/];
            });
        });
    };
    BroswerCompilation.prototype.compile = function (watch) {
        if (watch === void 0) { watch = false; }
        this.tscCompilation(this.compilationFolderPath, watch, "../" + this.backendOutFolder + "/" + this.outFolder, true, this.customCompiler ? this.customCompiler : void 0);
    };
    tslib_1.__decorate([
        incremental_compiler_1.IncCompiler.methods.AsyncAction(),
        tslib_1.__metadata("design:type", Function),
        tslib_1.__metadata("design:paramtypes", [incremental_compiler_1.IncCompiler.Change]),
        tslib_1.__metadata("design:returntype", Promise)
    ], BroswerCompilation.prototype, "asyncAction", null);
    BroswerCompilation = tslib_1.__decorate([
        incremental_compiler_1.IncCompiler.Class({ className: 'BroswerCompilation' }),
        tslib_1.__metadata("design:paramtypes", [String, String, String, String, String, String])
    ], BroswerCompilation);
    return BroswerCompilation;
}(compilation_backend_1.BackendCompilation));
exports.BroswerCompilation = BroswerCompilation;
function copyToBrowserSrcCodition(absoluteFilePath) {
    return !absoluteFilePath.endsWith('.backend.ts') && !absoluteFilePath.endsWith('.spec.ts');
}
//#endregion
//# sourceMappingURL=compilation-browser.js.map