"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
//#region @backend
var path = require("path");
var child = require("child_process");
var _ = require("lodash");
var config_1 = require("./config");
var helpers_1 = require("../helpers");
var compilation_browser_1 = require("./compilation-browser");
var compilation_backend_1 = require("./compilation-backend");
var IncrementalBuildProcess = /** @class */ (function () {
    function IncrementalBuildProcess(outFolder, relativeLocationToCwd, cwd, addStandaloneBrowserCompilation) {
        if (outFolder === void 0) { outFolder = 'dist'; }
        if (relativeLocationToCwd === void 0) { relativeLocationToCwd = 'src'; }
        if (cwd === void 0) { cwd = process.cwd(); }
        if (addStandaloneBrowserCompilation === void 0) { addStandaloneBrowserCompilation = true; }
        this.compileOnce = false;
        if (_.isString(outFolder) && _.isString(relativeLocationToCwd) && _.isString(cwd)) {
            this.backendCompilation = new compilation_backend_1.BackendCompilation(outFolder, relativeLocationToCwd, cwd);
            var browserOutFolder = config_1.config.folder.browser;
            this.browserCompilations = [];
            if (addStandaloneBrowserCompilation) {
                var browser = new compilation_browser_1.BroswerCompilation("tmp-src-" + outFolder + "-" + browserOutFolder, browserOutFolder, relativeLocationToCwd, cwd, outFolder);
                this.browserCompilations.push(browser);
            }
        }
    }
    IncrementalBuildProcess.prototype.browserTaksName = function (taskName, bc) {
        return "browser " + taskName + " in " + path.basename(bc.compilationFolderPath);
    };
    IncrementalBuildProcess.prototype.backendTaskName = function (taskName) {
        return taskName + " in " + path.basename(this.backendCompilation.compilationFolderPath);
    };
    IncrementalBuildProcess.prototype.recreateBrowserLinks = function (bc) {
        var outDistPath = path.join(bc.cwd, bc.outFolder);
        helpers_1.Helpers.System.Operations.tryRemoveDir(outDistPath);
        var targetOut = path.join(bc.cwd, bc.backendOutFolder, bc.outFolder);
        child.execSync(helpers_1.Helpers.createLink(outDistPath, targetOut));
    };
    IncrementalBuildProcess.prototype.start = function (taskName) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var _loop_1, this_1, index;
            var _this = this;
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        if (!this.compileOnce) {
                            this.compileOnce = true;
                        }
                        if (!this.backendCompilation) return [3 /*break*/, 2];
                        return [4 /*yield*/, this.backendCompilation.start(this.backendTaskName(taskName))];
                    case 1:
                        _a.sent();
                        _a.label = 2;
                    case 2:
                        _loop_1 = function (index) {
                            var browserCompilation;
                            return tslib_1.__generator(this, function (_a) {
                                switch (_a.label) {
                                    case 0:
                                        browserCompilation = this_1.browserCompilations[index];
                                        return [4 /*yield*/, browserCompilation.start(this_1.browserTaksName(taskName, browserCompilation), function () {
                                                _this.recreateBrowserLinks(browserCompilation);
                                            })];
                                    case 1:
                                        _a.sent();
                                        return [2 /*return*/];
                                }
                            });
                        };
                        this_1 = this;
                        index = 0;
                        _a.label = 3;
                    case 3:
                        if (!(index < this.browserCompilations.length)) return [3 /*break*/, 6];
                        return [5 /*yield**/, _loop_1(index)];
                    case 4:
                        _a.sent();
                        _a.label = 5;
                    case 5:
                        index++;
                        return [3 /*break*/, 3];
                    case 6: return [2 /*return*/];
                }
            });
        });
    };
    IncrementalBuildProcess.prototype.startAndWatch = function (taskName) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var _loop_2, this_2, index;
            var _this = this;
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        if (!this.compileOnce) return [3 /*break*/, 2];
                        console.log('Watch compilation single run');
                        return [4 /*yield*/, this.start(taskName)];
                    case 1:
                        _a.sent();
                        process.exit(0);
                        return [2 /*return*/];
                    case 2:
                        if (!this.backendCompilation) return [3 /*break*/, 4];
                        return [4 /*yield*/, this.backendCompilation.startAndWatch(this.backendTaskName(taskName))];
                    case 3:
                        _a.sent();
                        _a.label = 4;
                    case 4:
                        _loop_2 = function (index) {
                            var browserCompilation;
                            return tslib_1.__generator(this, function (_a) {
                                switch (_a.label) {
                                    case 0:
                                        browserCompilation = this_2.browserCompilations[index];
                                        return [4 /*yield*/, browserCompilation.startAndWatch(this_2.browserTaksName(taskName, browserCompilation), function () {
                                                _this.recreateBrowserLinks(browserCompilation);
                                            })];
                                    case 1:
                                        _a.sent();
                                        return [2 /*return*/];
                                }
                            });
                        };
                        this_2 = this;
                        index = 0;
                        _a.label = 5;
                    case 5:
                        if (!(index < this.browserCompilations.length)) return [3 /*break*/, 8];
                        return [5 /*yield**/, _loop_2(index)];
                    case 6:
                        _a.sent();
                        _a.label = 7;
                    case 7:
                        index++;
                        return [3 /*break*/, 5];
                    case 8: return [2 /*return*/];
                }
            });
        });
    };
    return IncrementalBuildProcess;
}());
exports.IncrementalBuildProcess = IncrementalBuildProcess;
//#endregion
//# sourceMappingURL=incremental-build-process.js.map