"use strict";
//#region @backend
Object.defineProperty(exports, "__esModule", { value: true });
var fse = require("fs-extra");
var path = require("path");
var _ = require("lodash");
var browser_code_cut_1 = require("./browser-code-cut");
var config_1 = require("./config");
exports.FILE_NAME_ISOMORPHIC_PACKAGES = 'tmp-isomorphic-packages.json';
var PackagesRecognition = /** @class */ (function () {
    function PackagesRecognition(cwd) {
        this.cwd = cwd;
    }
    PackagesRecognition.From = function (cwd) {
        return new PackagesRecognition(cwd);
    };
    Object.defineProperty(PackagesRecognition.prototype, "count", {
        get: function () {
            return _.isArray(this.recognizedPackages) ? this.recognizedPackages.length : 0;
        },
        enumerable: true,
        configurable: true
    });
    PackagesRecognition.prototype.start = function (force) {
        var _this = this;
        if (force === void 0) { force = false; }
        var pjPath = path.join(this.cwd, exports.FILE_NAME_ISOMORPHIC_PACKAGES);
        if (!fse.existsSync(pjPath)) {
            fse.writeJSONSync(pjPath, {}, { encoding: 'utf8' });
        }
        if (!force) {
            try {
                var pj = fse.readJSONSync(pjPath, {
                    encoding: 'utf8'
                });
                if (_.isArray(pj[config_1.config.array.isomorphicPackages])) {
                    this.recognizedPackages = pj[config_1.config.array.isomorphicPackages];
                    browser_code_cut_1.BrowserCodeCut.IsomorphicLibs = _.cloneDeep(this.recognizedPackages);
                    // console.log("RECOGNIZEDDDDD in ", pjPath)
                    return;
                }
            }
            catch (error) {
            }
        }
        var node_modules = path.join(this.cwd, config_1.config.folder.node_modules);
        var folders = fse.readdirSync(node_modules + "/");
        folders = folders.filter(function (packageName) {
            try {
                return _this.checkIsomorphic(node_modules, packageName);
            }
            catch (error) {
                return false;
            }
        });
        this.recognizedPackages = folders;
        this.updateCurrentPackageJson();
    };
    PackagesRecognition.prototype.updateCurrentPackageJson = function () {
        // console.log('updateCurrentPackageJsonupdateCurrentPackageJsonupdateCurrentPackageJson')
        try {
            var pjPath = path.join(this.cwd, exports.FILE_NAME_ISOMORPHIC_PACKAGES);
            if (!fse.existsSync(pjPath)) {
                fse.writeJSONSync(pjPath, {}, { encoding: 'utf8' });
            }
            var pj = fse.readJSONSync(pjPath, {
                encoding: 'utf8'
            });
            pj[config_1.config.array.isomorphicPackages] = this.recognizedPackages;
            fse.writeJSONSync(pjPath, pj, {
                encoding: 'utf8',
                spaces: 2
            });
            browser_code_cut_1.BrowserCodeCut.IsomorphicLibs = _.cloneDeep(this.recognizedPackages);
        }
        catch (e) {
            console.log(e);
            console.error("Error during update ismorphic packages list cache");
        }
    };
    PackagesRecognition.prototype.checkIsomorphic = function (node_modules, packageName) {
        var browser = path.join(node_modules, packageName, config_1.config.folder.browser);
        return fse.existsSync(browser);
    };
    return PackagesRecognition;
}());
exports.PackagesRecognition = PackagesRecognition;
//#endregion
//# sourceMappingURL=packages-recognition.js.map