'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});
exports.getFromlyConfigFor = getFromlyConfigFor;

var _lodash = require('lodash');

var _ = _interopRequireWildcard(_lodash);

var _browser = require('ng2-rest/browser');

var _typeFromEntity = require('./type-from-entity');

var _browser2 = require('typescript-class-helpers/browser');

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function getFromlyConfigFor(target) {
    var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
    var _options$formType = options.formType,
        formType = _options$formType === undefined ? 'material' : _options$formType,
        _options$keysPathesTo = options.keysPathesToExclude,
        keysPathesToExclude = _options$keysPathesTo === undefined ? [] : _options$keysPathesTo,
        _options$keysPathesTo2 = options.keysPathesToInclude,
        keysPathesToInclude = _options$keysPathesTo2 === undefined ? [] : _options$keysPathesTo2,
        parentModel = options.parentModel,
        _options$relativePath = options.relativePath,
        relativePath = _options$relativePath === undefined ? '' : _options$relativePath,
        _options$level = options.level,
        level = _options$level === undefined ? 0 : _options$level,
        _options$maxLevel = options.maxLevel,
        maxLevel = _options$maxLevel === undefined ? 4 : _options$maxLevel;

    if (_browser2.CLASS.getName(target) === 'Project') {
        return [];
    }
    if (level === maxLevel) {
        return [];
    }
    var mapping = _browser.Mapping.getModelsMapping(target);
    // console.log('mapping', mapping)
    var fieldNames = _browser2.CLASS.describeProperites(target);
    var checkExclude = _.isArray(keysPathesToExclude) && keysPathesToExclude.length > 0;
    var checkInclude = _.isArray(keysPathesToInclude) && keysPathesToInclude.length > 0;
    if (checkExclude && checkInclude) {
        throw 'In morphi function getFromlyConfigFor(...) please use keysPathesToInclude or keysPathesToExclude, ';
    }
    // if (checkInclude) {
    //   console.log('check include', keysPathesToExclude)
    // }
    // if (checkExclude) {
    //   console.log('check exclude', keysPathesToExclude)
    // }
    var fields = [];
    function inputToPush(key, type, model, targetChild) {
        // console.log(`key(${key}) type: ${type} | model: ${model} targetChild: ${targetChild && targetChild.name}`)
        var res = void 0;
        if (type === 'repeat') {
            var fieldGroup = getFromlyConfigFor(targetChild, {
                formType: formType, keysPathesToInclude: keysPathesToInclude, keysPathesToExclude: keysPathesToExclude,
                relativePath: relativePath + '.' + key,
                level: level + 1, maxLevel: maxLevel
            });
            if (fieldGroup.length > 0) {
                res = {
                    key: key,
                    type: type,
                    defaultValue: [],
                    fieldArray: {
                        fieldGroupClassName: 'row',
                        templateOptions: {
                            label: 'Add new ' + _.startCase(key)
                        },
                        fieldGroup: fieldGroup
                    }
                };
            }
        } else if (type === 'group') {
            var _fieldGroup = getFromlyConfigFor(targetChild, {
                formType: formType, keysPathesToInclude: keysPathesToInclude, keysPathesToExclude: keysPathesToExclude, parentModel: model,
                relativePath: relativePath + '.' + key,
                level: level + 1, maxLevel: maxLevel
            });
            if (_fieldGroup.length > 0) {
                res = {
                    fieldGroupClassName: 'row',
                    templateOptions: {
                        label: '' + _.startCase(key)
                    },
                    wrappers: ['groupwrap'],
                    fieldGroup: _fieldGroup
                };
            }
        } else {
            res = {
                key: key,
                model: model,
                type: type,
                defaultValue: !_.isUndefined(target.prototype[key]) ? target.prototype[key] : undefined,
                templateOptions: {
                    label: _.isString(model) ? model.split('.').map(function (l) {
                        return _.startCase(l);
                    }).join(' / ') + ' / ' + _.startCase(key) : _.startCase(key)
                }
            };
        }
        if (res) {
            Object.keys(res).forEach(function (key) {
                return res[key] === undefined ? delete res[key] : '';
            });
        }
        return res;
    }
    function isAlowedPath(key) {
        var isAlowed = true;
        var matchPath = relativePath === '' ? key : relativePath + ':' + key;
        if (checkExclude) {
            if (keysPathesToExclude.includes(matchPath)) {
                // console.log(`Not allowed key: ${key}`)
                isAlowed = false;
            } else {
                isAlowed = true;
            }
        } else if (checkInclude) {
            if (keysPathesToInclude.includes(matchPath)) {
                // console.log(`Allowed key: ${key}`)
                isAlowed = true;
            } else {
                isAlowed = false;
            }
        }
        // console.log(`Is allowed;${matchPath} `, isAlowed)
        return isAlowed;
    }
    var simpleResolved = [];
    function resolveSimpleTypes() {
        for (var key in target.prototype) {
            if (target.prototype.hasOwnProperty(key) && !_.isFunction(target.prototype[key])) {
                if (!isAlowedPath(key)) {
                    continue;
                }
                if (!_.isUndefined(mapping[key])) {
                    continue;
                }
                var element = target.prototype[key];
                var type = 'input';
                if (_.isBoolean(element)) {
                    type = 'switch';
                } else if (_.isDate(element)) {
                    type = 'datepicker';
                }
                fields.push(inputToPush(key, type, parentModel));
                simpleResolved.push(key);
            }
        }
    }
    function resolveComplexTypes() {
        fieldNames.filter(function (key) {
            return !simpleResolved.includes(key);
        }).forEach(function (key) {
            if (isAlowedPath(key) && !_.isUndefined(mapping[key])) {
                var className = mapping[key];
                var isArray = _.isArray(className);
                className = isArray ? _.first(className) : className;
                if (className === 'Date') {
                    fields.push(inputToPush(key, 'datepicker', parentModel));
                } else {
                    var targetChild = _browser2.CLASS.getBy(className);
                    if (targetChild) {
                        var ftype = (0, _typeFromEntity.findTypeForEntity)(targetChild, isArray);
                        if (ftype) {
                            fields = fields.concat(inputToPush(key, ftype.name, key));
                        } else {
                            if (isArray) {
                                fields = fields.concat(inputToPush(key, 'repeat', key, targetChild));
                            } else {
                                fields = fields.concat(inputToPush(key, 'group', key, targetChild));
                            }
                        }
                    }
                }
            }
        });
    }
    function generate() {
        resolveSimpleTypes();
        resolveComplexTypes();
    }
    generate();
    return fields.filter(function (f) {
        return !!f;
    });
}