'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});
exports.Providers = exports.Connection = undefined;

var _browser = require('typeorm/browser');

Object.defineProperty(exports, 'Connection', {
    enumerable: true,
    get: function get() {
        return _browser.Connection;
    }
});
exports.init = init;

var _lodash = require('lodash');

var _ = _interopRequireWildcard(_lodash);

var _globalConfig = require('./global-config');

var _realtime = require('./realtime');

var _symbols = require('./symbols');

var _helpers = require('./helpers');

var _browser2 = require('typescript-class-helpers/browser');

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function init(config) {
    var ngZone = config.ngZone,
        _config$controllers = config.controllers,
        controllers = _config$controllers === undefined ? [] : _config$controllers,
        _config$entities = config.entities,
        entities = _config$entities === undefined ? [] : _config$entities,
        _config$productionMod = config.productionMode,
        productionMode = _config$productionMod === undefined ? false : _config$productionMod,
        _config$allowedHosts = config.allowedHosts,
        allowedHosts = _config$allowedHosts === undefined ? [] : _config$allowedHosts;

    if (_.isArray(controllers) && controllers.filter(function (f) {
        return !_.isFunction(f);
    }).length > 0) {
        console.error('controllers', controllers);
        throw '\n\nIncorect value for property "controllers" inside Morphi.Init(...)\n\n';
    }
    if (_.isArray(entities) && entities.filter(function (f) {
        return !_.isFunction(f);
    }).length > 0) {
        console.error('entites', entities);
        throw '\n\nIncorect value for property "entities" inside Morphi.Init(...)\n\n';
    }
    if (_helpers.Helpers.isBrowser && _.isUndefined(ngZone) && !!window['ng']) {
        console.warn('Please probide ngZone instance in angular apps');
    }
    _globalConfig.Global.vars.ngZone = ngZone;
    _globalConfig.Global.vars.entities = config.entities;
    _globalConfig.Global.vars.controllers = config.controllers;
    _globalConfig.Global.vars.__core_controllers.forEach(function (bctrl) {
        return controllers.push(bctrl);
    });
    config.controllers = _.sortedUniq(controllers);
    _globalConfig.Global.vars.productionMode = !!config.productionMode;
    // backend URI URL {
    //   href: 'http://localhost:4000/api',
    //   origin: 'http://localhost:4000',
    //   protocol: 'http:',
    //   username: '',
    //   password: '',
    //   host: 'localhost:4000',
    //   hostname: 'localhost',
    //   port: '4000',
    //   pathname: '/api',
    //   search: '',
    //   searchParams: URLSearchParams {},
    //   hash: '' }
    if (_helpers.Helpers.isBrowser) {
        var uri = new URL(config.host);
        _globalConfig.Global.vars.url = uri;
        if (Array.isArray(allowedHosts)) {
            _globalConfig.Global.vars.allowedHosts = allowedHosts.map(function (h) {
                return new URL(h);
            });
        }
        _realtime.RealtimeBrowser.init();
    }
    if (_helpers.Helpers.isBrowser) {
        var notFound = [];
        var providers = controllers.filter(function (ctrl) {
            var e = _globalConfig.Global.vars.initFunc.find(function (e) {
                return ctrl === e.target;
            });
            if (e) {
                // console.log('current controller ', currentCtrl)
                e.initFN();
                return true;
            } else {
                var context = ctrl.prototype[_symbols.SYMBOL.CLASS_DECORATOR_CONTEXT];
                if (!context) {
                    notFound.push(ctrl);
                    return false;
                } else {
                    context.initFN();
                    return true;
                }
            }
        });
        notFound.forEach(function (ctrl) {
            throw 'Decorator "@ENDPOINT(..)" is missing on class ' + _browser2.CLASS.getName(ctrl);
        });
        providers.forEach(function (p) {
            return Providers.push(p);
        });
    }
}
var Providers = exports.Providers = [];