"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
//#region @backend
require("reflect-metadata");
var typeorm_1 = require("typeorm");
exports.Connection = typeorm_1.Connection;
var symbols_1 = require("../symbols");
var typescript_class_helpers_1 = require("typescript-class-helpers");
//#endregion
var _ = require("lodash");
var global_config_1 = require("../global-config");
//#region @backend
function tableNameFrom(entityClass) {
    entityClass = entityClass;
    return "tb_" + entityClass.name.toLowerCase();
}
exports.tableNameFrom = tableNameFrom;
//#endregion
function classNameVlidation(className, target) {
    if (_.isUndefined(className)) {
        if (global_config_1.Global.vars.isProductionMode) {
            throw "[Morphi] Please provide \"className\" property for each Controller and Entity:\n\n      @Morphi.Controller({ className: 'MyExampleCtrl'  })\n      class MyExampleCtrl {\n        ...\n      }\n\n      @Morphi.Entity({ className: 'MyExampleEntity'  })\n      class MyExampleEntity {\n        ...\n      }\n    ";
        }
        className = target.name;
    }
    return className;
}
exports.classNameVlidation = classNameVlidation;
//#region @backend
function repositoryFrom(connection, entityFN, repoFn) {
    if (!connection) {
        console.error("[Morphi][repositoryFrom] no connection!\nPlease check your Morphi.Repository(...) decorators ", entityFN, repoFn);
        return;
    }
    var repo;
    if (!!entityFN && !entityFN[symbols_1.SYMBOL.HAS_TABLE_IN_DB]) {
        if (_.isFunction(repoFn)) {
            var repo_1 = typescript_class_helpers_1.CLASS.getSingleton(repoFn);
            if (!repo_1) {
                typescript_class_helpers_1.CLASS.setSingletonObj(repoFn, new repoFn());
            }
            repo_1 = typescript_class_helpers_1.CLASS.getSingleton(repoFn);
            return repo_1;
        }
        console.warn("Repository function not abailable for " + typescript_class_helpers_1.CLASS.getName(entityFN));
        return;
    }
    if (repoFn) {
        repo = connection.getCustomRepository(repoFn);
        var existedRepo = typescript_class_helpers_1.CLASS.getSingleton(repoFn);
        if (!existedRepo) {
            typescript_class_helpers_1.CLASS.setSingletonObj(repoFn, repo);
        }
    }
    else {
        repo = connection.getRepository(entityFN);
    }
    repo['_'] = {};
    repo['__'] = {};
    var compolexProperties = repo.globalAliases;
    if (Array.isArray(compolexProperties)) {
        compolexProperties.forEach(function (alias) {
            repo['__'][alias] = {};
            var describedProps = typescript_class_helpers_1.CLASS.describeProperites(entityFN);
            // console.log(`describedProps  "${describedProps}" for ${entity.name}`)
            describedProps.concat(compolexProperties).forEach(function (prop) {
                repo['__'][alias][prop] = alias + "." + prop; // TODO temp solution
            });
            var props = typescript_class_helpers_1.CLASS.describeProperites(entityFN);
            // console.log(`props  "${props}" for ${entity.name}`)
            props.forEach(function (prop) {
                repo['__'][alias][prop] = alias + "." + prop; // TODO ideal solution
            });
        });
        compolexProperties.forEach(function (alias) {
            repo['_'][alias] = alias; // TODO make it getter with reference
        });
    }
    return repo;
}
exports.repositoryFrom = repositoryFrom;
//#endregion
//# sourceMappingURL=framework-helpers.js.map