"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
//#region @backend
var path = require("path");
var child = require("child_process");
var os = require("os");
var fs = require("fs");
var fse = require("fs-extra");
var chalk_1 = require("chalk");
var rimraf = require("rimraf");
var dateformat = require("dateformat");
var commandExistsSync = require('command-exists').sync;
//#endregion
var JSON5 = require("json5");
var _ = require("lodash");
var ng2_rest_1 = require("ng2-rest");
var symbols_1 = require("./symbols");
//#endregion
var typescript_class_helpers_1 = require("typescript-class-helpers");
var Helpers = /** @class */ (function (_super) {
    tslib_1.__extends(Helpers, _super);
    function Helpers() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    Object.defineProperty(Helpers, "System", {
        //#region @backend
        get: function () {
            return {
                get Operations() {
                    return {
                        tryRemoveDir: function (dirpath) {
                            rimraf.sync(dirpath);
                        },
                        tryCopyFrom: function (source, destination, options) {
                            if (options === void 0) { options = {}; }
                            // console.log(`Trying to copy from hahah: ${source} to ${destination}`)
                            fse.copySync(source, destination, _.merge({
                                overwrite: true,
                                recursive: true
                            }, options));
                        }
                    };
                }
            };
        },
        enumerable: true,
        configurable: true
    });
    //#endregion
    Helpers.isGoodPath = function (p) {
        return p && typeof p === 'string' && p.trim() !== '';
    };
    Helpers.isRealtimeEndpoint = function (target) {
        return target && target.prototype && target.prototype[symbols_1.SYMBOL.IS_ENPOINT_REALTIME];
    };
    Helpers.hasParentClassWithName = function (target, name, targets) {
        if (targets === void 0) { targets = []; }
        if (!target) {
            // console.log(`false "${_.first(targets).name}" for ${targets.map(d => d.name).join(',')}`)
            return false;
        }
        targets.push(target);
        var targetProto = target['__proto__'];
        if (_.isFunction(targetProto) && typescript_class_helpers_1.CLASS.getName(targetProto) === name) {
            // console.log(`true  "${_.first(targets).name}" for ${targets.map(d => d.name).join(',')}`)
            return true;
        }
        return this.hasParentClassWithName(targetProto, name, targets);
    };
    //#region @backend
    Helpers.compilationWrapper = function (fn, taskName, executionType) {
        if (taskName === void 0) { taskName = 'Task'; }
        if (executionType === void 0) { executionType = 'Compilation'; }
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            function currentDate() {
                return "[" + dateformat(new Date(), 'HH:MM:ss') + "]";
            }
            var error_1;
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        if (!fn || !_.isFunction(fn)) {
                            console.error(executionType + " wrapper: \"" + fs + "\" is not a function.");
                            process.exit(1);
                        }
                        _a.label = 1;
                    case 1:
                        _a.trys.push([1, 3, , 4]);
                        console.log(chalk_1.default.gray(currentDate() + " " + executionType + " of \"" + chalk_1.default.bold(taskName) + "\" started..."));
                        return [4 /*yield*/, Helpers.runSyncOrAsync(fn)];
                    case 2:
                        _a.sent();
                        console.log(chalk_1.default.green(currentDate() + " " + executionType + " of \"" + chalk_1.default.bold(taskName) + "\" finish OK..."));
                        return [3 /*break*/, 4];
                    case 3:
                        error_1 = _a.sent();
                        console.log(chalk_1.default.red(error_1));
                        console.log(currentDate() + " " + executionType + " of " + taskName + " ERROR");
                        return [3 /*break*/, 4];
                    case 4: return [2 /*return*/];
                }
            });
        });
    };
    //#endregion
    Helpers.runSyncOrAsync = function (fn) {
        // let wasPromise = false;
        var promisOrValue = fn();
        if (promisOrValue instanceof Promise) {
            // wasPromise = true;
            promisOrValue = Promise.resolve(promisOrValue);
        }
        // console.log('was promis ', wasPromise)
        return promisOrValue;
    };
    Helpers.tryTransformParam = function (param) {
        if (typeof param === 'string') {
            var n = Number(param);
            if (!isNaN(n))
                return n;
            var bool = param.trim().toLowerCase();
            if (bool === 'true') {
                return true;
            }
            if (bool === 'false') {
                return false;
            }
            try {
                var t = JSON5.parse(param);
                return t;
            }
            catch (e) {
                return param;
            }
        }
        return param;
    };
    Helpers.getExpressPath = function (c, pathOrClassConfig) {
        if (typeof pathOrClassConfig === 'string')
            return ("" + c.calculatedPath + pathOrClassConfig).replace(/\/$/, '');
        return ("" + c.calculatedPath + pathOrClassConfig.path).replace(/\/$/, '');
    };
    Helpers.defaultType = function (value) {
        if (typeof value === 'string')
            return '';
        if (typeof value === 'boolean')
            return false;
        if (Array.isArray(value))
            return {};
        if (typeof value === 'object')
            return {};
    };
    Helpers.getSingleton = function (target) {
        var configs = typescript_class_helpers_1.CLASS.getConfig(target);
        return ((Array.isArray(configs) && configs.length >= 1) ? configs[0].singleton : undefined);
    };
    Helpers.getSingletons = function (target) {
        var configs = typescript_class_helpers_1.CLASS.getConfig(target);
        return configs.map(function (c) { return c.singleton; });
    };
    Helpers.parseJSONwithStringJSONs = function (object, waring) {
        var _this = this;
        if (waring === void 0) { waring = false; }
        // console.log('checking object', object)
        if (!_.isObject(object)) {
            if (waring) {
                console.error("\n        parseJSONwithStringJSONs(...)\n        Parameter should be a object, but is " + typeof object + "\n        ", object);
            }
            return object;
        }
        var res = _.cloneDeep(object);
        Object.keys(res).forEach(function (key) {
            var isJson = false;
            try {
                var possibleJSON = JSON.parse(res[key]);
                res[key] = possibleJSON;
                isJson = true;
            }
            catch (e) {
                isJson = false;
            }
            // console.log(`key ${key} is json `, isJson)
            if (isJson) {
                res[key] = _this.parseJSONwithStringJSONs(res[key], false);
            }
        });
        return res;
    };
    //#region @backend
    Helpers.getResponseValue = function (response, req, res) {
        var _this = this;
        //#region @backendFunc
        return new Promise(function (resolve, reject) { return tslib_1.__awaiter(_this, void 0, void 0, function () {
            var resp, asyncResponse, result, e_1, result;
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        resp = response;
                        if (!(!response && response.send === undefined)) return [3 /*break*/, 1];
                        console.error('Bad response value for function');
                        resolve(undefined);
                        return [3 /*break*/, 7];
                    case 1:
                        if (!(typeof response === 'function')) return [3 /*break*/, 6];
                        asyncResponse = response;
                        _a.label = 2;
                    case 2:
                        _a.trys.push([2, 4, , 5]);
                        return [4 /*yield*/, asyncResponse(req, res)];
                    case 3:
                        result = _a.sent();
                        resolve(result);
                        return [3 /*break*/, 5];
                    case 4:
                        e_1 = _a.sent();
                        if (e_1 && e_1.stack) {
                            console.log(e_1.stack);
                        }
                        console.error('Bad async function call ', e_1);
                        reject(e_1);
                        return [3 /*break*/, 5];
                    case 5: return [3 /*break*/, 7];
                    case 6:
                        if (typeof response === 'object') {
                            try {
                                if (typeof response.send === 'function') {
                                    result = response.send(req, res);
                                    resolve(result);
                                }
                                else {
                                    resolve(response.send);
                                }
                            }
                            catch (error) {
                                console.error('Bad synchonus function call ', error);
                                reject(error);
                            }
                        }
                        else
                            reject("Not recognized type of reposne " + response);
                        _a.label = 7;
                    case 7: return [2 /*return*/];
                }
            });
        }); });
        //#endregion
    };
    Helpers.checkEnvironment = function (globalDependencies) {
        if (globalDependencies === void 0) { globalDependencies = this.MorphiGlobalDependencies; }
        var missingNpm = [];
        globalDependencies.npm.forEach(function (pkg) {
            if (!commandExistsSync(pkg.name)) {
                missingNpm.push(pkg);
            }
        });
        if (missingNpm.length > 0) {
            var toInstall = missingNpm
                .map(function (pkg) {
                var n = pkg.installName ? pkg.installName : pkg.name;
                return pkg.version ? n + "@" + pkg.version : n;
            })
                .join(' ');
            console.log(chalk_1.default.red("Missing npm dependencies."));
            var cmd = "npm install -g " + toInstall;
            console.log("Please run: " + chalk_1.default.green(cmd));
            process.exit(0);
        }
        globalDependencies.programs.forEach(function (p) {
            if (!commandExistsSync(p.name)) {
                console.log(chalk_1.default.red("Missing command line tool \"" + p.name + "\"."));
                console.log("Please install it from: " + chalk_1.default.green(p.website));
                process.exit(0);
            }
        });
        try {
            child.execSync("check-node-version --node \">= 9.2\"", { stdio: [0, 1, 2] });
        }
        catch (error) {
            process.exit(0);
        }
    };
    Helpers.isPlainFileOrFolder = function (filePath) {
        return /^([a-zA-Z]|\-|\_|\@|\#|\$|\!|\^|\&|\*|\(|\))+$/.test(filePath);
    };
    Helpers.log = function (proc) {
        // processes.push(proc);
        // let stdio = [0,1,2]
        proc.stdout.on('data', function (data) {
            process.stdout.write(data);
            // console.log(data.toString());
        });
        proc.stdout.on('error', function (data) {
            process.stdout.write(JSON.stringify(data));
            // console.log(data);
        });
        proc.stderr.on('data', function (data) {
            process.stderr.write(data);
            // console.log(data.toString());
        });
        proc.stderr.on('error', function (data) {
            process.stderr.write(JSON.stringify(data));
            // console.log(data);
        });
    };
    Helpers.createLink = function (target, link) {
        if (this.isPlainFileOrFolder(link)) {
            link = path.join(process.cwd(), link);
        }
        var command;
        if (os.platform() === 'win32') {
            if (target.startsWith('./')) {
                target = path.win32.normalize(path.join(process.cwd(), path.basename(target)));
            }
            else {
                if (target === '.' || target === './') {
                    target = path.win32.normalize(path.join(process.cwd(), path.basename(link)));
                }
                else {
                    target = path.win32.normalize(path.join(target, path.basename(link)));
                }
            }
            if (fs.existsSync(target)) {
                fs.unlinkSync(target);
            }
            target = path.win32.normalize(target);
            if (link === '.' || link === './') {
                link = process.cwd();
            }
            link = path.win32.normalize(link);
            // console.log('taget', target)
            // console.log('link', link)
            command = "mklink \/D "
                + target
                + " "
                + link
                + " >nul 2>&1 ";
            // console.log('LINK COMMAND', command)
        }
        else {
            if (target.startsWith('./')) {
                target = target.replace(/^\.\//g, '');
            }
            if (link === '.' || link === './') {
                link = process.cwd();
            }
            command = "ln -sf \"" + link + "\" \"" + target + "\"";
        }
        // console.log(command)
        return command;
    };
    Helpers.getRecrusiveFilesFrom = function (dir) {
        var _this = this;
        var files = [];
        var readed = fs.readdirSync(dir).map(function (f) {
            var fullPath = path.join(dir, f);
            // console.log(`is direcotry ${fs.lstatSync(fullPath).isDirectory()} `, fullPath)
            if (fs.lstatSync(fullPath).isDirectory()) {
                _this.getRecrusiveFilesFrom(fullPath).forEach(function (aa) { return files.push(aa); });
            }
            return fullPath;
        });
        if (Array.isArray(readed)) {
            readed.forEach(function (r) { return files.push(r); });
        }
        return files;
    };
    Helpers.MorphiGlobalDependencies = {
        npm: [
            { name: 'rimraf' },
            { name: 'npm-run' },
            { name: 'cpr' },
            { name: 'check-node-version' }
        ],
        programs: [
        // {
        //   name: 'code',
        //   website: 'https://code.visualstudio.com/'
        // }
        ]
    };
    return Helpers;
}(ng2_rest_1.Helpers));
exports.Helpers = Helpers;
//# sourceMappingURL=helpers.js.map