"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var _ = require("lodash");
var models_1 = require("../models");
var helpers_1 = require("../helpers");
var global_config_1 = require("../global-config");
var symbols_1 = require("../symbols");
//#region @backend
var cors = require("cors");
var bodyParser = require("body-parser");
var cookieParser = require("cookie-parser");
var methodOverride = require("method-override");
var fileUpload = require("express-fileupload");
var entity_process_1 = require("./entity-process");
var crud_1 = require("../crud");
//#endregion
// function mesureTime(func: (note: (info: string) => any) => any) {
//   let start = process.hrtime();
//   const elapsed_time = (note) => {
//     var precision = 3; // 3 decimal places
//     var elapsed = process.hrtime(start)[1] / 1000000; // divide by a million to get nano to milli
//     console.log(process.hrtime(start)[0] + " s, " + elapsed.toFixed(precision) + " ms - " + note); // print message + time
//     start = process.hrtime(); // reset the timer
//   }
//   func(elapsed_time as any)
// }
function initMidleware() {
    //#region @backend
    var app = global_config_1.Global.vars.app;
    app.use(fileUpload());
    app.use(bodyParser.urlencoded({ extended: true }));
    app.use(bodyParser.json());
    app.use(methodOverride());
    app.use(cookieParser());
    app.use(cors());
    (function () {
        app.use(function (req, res, next) {
            res.set('Access-Control-Expose-Headers', [
                'Content-Type',
                'Authorization',
                'X-Requested-With',
                symbols_1.SYMBOL.X_TOTAL_COUNT,
                symbols_1.SYMBOL.MAPPING_CONFIG_HEADER,
                symbols_1.SYMBOL.CIRCURAL_OBJECTS_MAP_BODY,
                symbols_1.SYMBOL.CIRCURAL_OBJECTS_MAP_QUERY_PARAM
            ].join(', '));
            next();
        });
    })();
    //#endregion
}
exports.initMidleware = initMidleware;
//#region @backend
function initMethodNodejs(type, methodConfig, classConfig, expressPath) {
    var _this = this;
    var requestHandler = (methodConfig.requestHandler && typeof methodConfig.requestHandler === 'function')
        ? methodConfig.requestHandler : function (req, res, next) { next(); };
    var productionMode = global_config_1.Global.vars.productionMode;
    global_config_1.Global.vars.url.pathname = global_config_1.Global.vars.url.pathname.replace(/\/$/, "");
    expressPath = global_config_1.Global.vars.url.pathname.startsWith('/') ? "" + global_config_1.Global.vars.url.pathname + expressPath : expressPath;
    expressPath = expressPath.replace(/\/\//g, '/');
    // console.log(`BACKEND: expressPath: ${expressPath}`)
    global_config_1.Global.vars.app[type.toLowerCase()](expressPath, requestHandler, function (req, res) { return tslib_1.__awaiter(_this, void 0, void 0, function () {
        var args, tBody, tParams, tQuery, entity, entity, resolvedParams, response, result, mdc, error_1, err, err;
        return tslib_1.__generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    args = [];
                    tBody = req.body;
                    tParams = req.params;
                    tQuery = req.query;
                    if (req.headers[symbols_1.SYMBOL.CIRCURAL_OBJECTS_MAP_BODY]) {
                        try {
                            tBody = helpers_1.Helpers.JSON.parse(JSON.stringify(tBody), JSON.parse(req.headers[symbols_1.SYMBOL.CIRCURAL_OBJECTS_MAP_BODY]));
                        }
                        catch (e) { }
                    }
                    if (req.headers[symbols_1.SYMBOL.CIRCURAL_OBJECTS_MAP_QUERY_PARAM]) {
                        try {
                            tQuery = helpers_1.Helpers.JSON.parse(JSON.stringify(tQuery), JSON.parse(req.headers[symbols_1.SYMBOL.CIRCURAL_OBJECTS_MAP_QUERY_PARAM]));
                        }
                        catch (e) { }
                    }
                    // make class instance from body
                    // console.log('req.headers', req.headers)
                    if (req.headers[symbols_1.SYMBOL.MAPPING_CONFIG_HEADER_BODY_PARAMS]) {
                        try {
                            entity = JSON.parse(req.headers[symbols_1.SYMBOL.MAPPING_CONFIG_HEADER_BODY_PARAMS]);
                            tBody = helpers_1.Helpers.Mapping.encode(tBody, entity);
                        }
                        catch (e) { }
                    }
                    else {
                        Object.keys(tBody).forEach(function (paramName) {
                            try {
                                var entityForParam = JSON.parse(req.headers["" + symbols_1.SYMBOL.MAPPING_CONFIG_HEADER_BODY_PARAMS + paramName]);
                                tBody[paramName] = helpers_1.Helpers.Mapping.encode(tBody[paramName], entityForParam);
                            }
                            catch (e) { }
                        });
                    }
                    // make class instance from query params
                    // console.log('req.headers', tQuery)
                    if (req.headers[symbols_1.SYMBOL.MAPPING_CONFIG_HEADER_QUERY_PARAMS]) {
                        try {
                            entity = JSON.parse(req.headers[symbols_1.SYMBOL.MAPPING_CONFIG_HEADER_QUERY_PARAMS]);
                            tQuery = helpers_1.Helpers.parseJSONwithStringJSONs(helpers_1.Helpers.Mapping.encode(tQuery, entity));
                        }
                        catch (e) { }
                    }
                    else {
                        Object.keys(tQuery).forEach(function (queryParamName) {
                            try {
                                var entityForParam = JSON.parse(req.headers["" + symbols_1.SYMBOL.MAPPING_CONFIG_HEADER_QUERY_PARAMS + queryParamName]);
                                var beforeTransofrm = tQuery[queryParamName];
                                if (_.isString(beforeTransofrm)) {
                                    try {
                                        var paresed = helpers_1.Helpers.tryTransformParam(beforeTransofrm);
                                        beforeTransofrm = paresed;
                                    }
                                    catch (e) { }
                                }
                                var afterEncoding = helpers_1.Helpers.Mapping.encode(beforeTransofrm, entityForParam);
                                tQuery[queryParamName] = helpers_1.Helpers.parseJSONwithStringJSONs(afterEncoding);
                            }
                            catch (e) { }
                        });
                    }
                    Object.keys(methodConfig.parameters).forEach(function (paramName) {
                        var p = methodConfig.parameters[paramName];
                        if (p.paramType === 'Path' && tParams) {
                            args.push(tParams[p.paramName]);
                        }
                        if (p.paramType === 'Query' && tQuery) {
                            if (p.paramName) {
                                args.push(tQuery[p.paramName]);
                            }
                            else {
                                args.push(tQuery);
                            }
                        }
                        if (p.paramType === 'Header' && req.headers) {
                            args.push(req.headers[p.paramName.toLowerCase()]);
                        }
                        if (p.paramType === 'Cookie' && req.cookies) {
                            args.push(req.cookies[p.paramName]);
                        }
                        if (p.paramType === 'Body' && tBody) {
                            if (p.paramName && typeof tBody === 'object') {
                                args.push(tBody[p.paramName]);
                            }
                            else {
                                args.push(tBody);
                            }
                        }
                    });
                    resolvedParams = args.reverse().map(function (v) { return helpers_1.Helpers.tryTransformParam(v); });
                    _a.label = 1;
                case 1:
                    _a.trys.push([1, 4, , 5]);
                    response = methodConfig.descriptor.value.apply(classConfig.singleton, resolvedParams);
                    return [4 /*yield*/, helpers_1.Helpers.getResponseValue(response, req, res)];
                case 2:
                    result = _a.sent();
                    mdc = crud_1.MDC.fromHeader(req);
                    // console.log(mdc)
                    return [4 /*yield*/, entity_process_1.EntityProcess.init(result, res, mdc)];
                case 3:
                    // console.log(mdc)
                    _a.sent();
                    return [3 /*break*/, 5];
                case 4:
                    error_1 = _a.sent();
                    if (_.isString(error_1)) {
                        res.status(400).send(helpers_1.Helpers.JSON.stringify({
                            message: "\nError inside: " + req.path + "\n\n" + error_1 + "\n\n"
                        }));
                    }
                    else if (error_1 instanceof models_1.Models.Errors) {
                        console.log('Morphi Error', error_1);
                        err = error_1;
                        res.status(400).send(helpers_1.Helpers.JSON.stringify(err));
                    }
                    else if (error_1 instanceof Error) {
                        err = error_1;
                        console.log('Code Error', error_1);
                        betterError(err);
                        res.status(400).send(helpers_1.Helpers.JSON.stringify({
                            stack: err.stack,
                            message: err.message
                        }));
                    }
                    else {
                        console.log("Bad result isomorphic method: " + error_1);
                        res.status(400).send(helpers_1.Helpers.JSON.stringify(error_1));
                    }
                    return [3 /*break*/, 5];
                case 5: return [2 /*return*/];
            }
        });
    }); });
    return {
        routePath: expressPath,
        method: methodConfig.type
    };
}
exports.initMethodNodejs = initMethodNodejs;
//#endregion
//#region @backend
function betterError(error) {
    console.log(require('callsite-record')({
        forError: error
    }).renderSync({
    // stackFilter(frame) {
    //   return !frame.getFileName().includes('node_modules');
    // }
    }));
}
//#endregion
//# sourceMappingURL=init-method-node.js.map