"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
//#region @backend
var express = require("express");
var http = require("http");
//#endregion
var _ = require("lodash");
var global_config_1 = require("./global-config");
var init_method_1 = require("./init-method");
var typeorm_1 = require("typeorm");
exports.Connection = typeorm_1.Connection;
var realtime_1 = require("./realtime");
var symbols_1 = require("./symbols");
var helpers_1 = require("./helpers");
var typescript_class_helpers_1 = require("typescript-class-helpers");
function init(config) {
    var ngZone = config.ngZone, _a = config.controllers, controllers = _a === void 0 ? [] : _a, _b = config.entities, entities = _b === void 0 ? [] : _b, _c = config.productionMode, productionMode = _c === void 0 ? false : _c, _d = config.allowedHosts, allowedHosts = _d === void 0 ? [] : _d, 
    //#region @backend
    _e = config.testMode, 
    //#region @backend
    testMode = _e === void 0 ? false : _e, connection = config.connection
    //#endregion
    ;
    if (_.isArray(controllers) && controllers.filter(function (f) { return !_.isFunction(f); }).length > 0) {
        console.error('controllers', controllers);
        throw "\n\nIncorect value for property \"controllers\" inside Morphi.Init(...)\n\n";
    }
    if (_.isArray(entities) && entities.filter(function (f) { return !_.isFunction(f); }).length > 0) {
        console.error('entites', entities);
        throw "\n\nIncorect value for property \"entities\" inside Morphi.Init(...)\n\n";
    }
    //#region @backend
    if (helpers_1.Helpers.isNode) {
        var URL = require('url').URL;
    }
    //#endregion
    if (helpers_1.Helpers.isBrowser && _.isUndefined(ngZone) && !!window['ng']) {
        console.warn("Please probide ngZone instance in angular apps");
    }
    global_config_1.Global.vars.ngZone = ngZone;
    global_config_1.Global.vars.entities = config.entities;
    global_config_1.Global.vars.controllers = config.controllers;
    global_config_1.Global.vars.__core_controllers.forEach(function (bctrl) { return controllers.push(bctrl); });
    config.controllers = _.sortedUniq(controllers);
    global_config_1.Global.vars.productionMode = !!config.productionMode;
    // backend URI URL {
    //   href: 'http://localhost:4000/api',
    //   origin: 'http://localhost:4000',
    //   protocol: 'http:',
    //   username: '',
    //   password: '',
    //   host: 'localhost:4000',
    //   hostname: 'localhost',
    //   port: '4000',
    //   pathname: '/api',
    //   search: '',
    //   searchParams: URLSearchParams {},
    //   hash: '' }
    //#region @backend
    if (helpers_1.Helpers.isNode) {
        if (!global_config_1.Global.vars.app) {
            global_config_1.Global.vars.app = express();
            init_method_1.initMidleware();
        }
        var uri_1 = new URL(config.host);
        // console.log('backend URI', uri);
        global_config_1.Global.vars.url = uri_1;
        // if (uri.pathname !== '/') {
        //   console.log('INT EXPRESS BASE')
        //   Global.vars.app.set('base', uri.pathname)
        // }
        var h = new http.Server(global_config_1.Global.vars.app); //TODO is this working ?
        realtime_1.RealtimeNodejs.init(h);
        if (!testMode) {
            h.listen(uri_1.port, function () {
                console.log("Server listening on port: " + uri_1.port + ", hostname: " + uri_1.pathname + ",\n          env: " + global_config_1.Global.vars.app.settings.env + "\n          ");
            });
        }
    }
    //#endregion
    if (helpers_1.Helpers.isBrowser) {
        var uri = new URL(config.host);
        global_config_1.Global.vars.url = uri;
        if (Array.isArray(allowedHosts)) {
            global_config_1.Global.vars.allowedHosts = allowedHosts.map(function (h) { return new URL(h); });
        }
        realtime_1.RealtimeBrowser.init();
    }
    //#region @backend
    if (helpers_1.Helpers.isNode) {
        global_config_1.Global.vars.connection = connection;
        global_config_1.Global.vars.initFunc.filter(function (e) {
            var currentCtrl = controllers.find(function (ctrl) { return ctrl === e.target; });
            if (currentCtrl) {
                e.initFN();
                (function (controller) {
                    var c = typescript_class_helpers_1.CLASS.getConfig(currentCtrl)[0];
                    c.injections.forEach(function (inj) {
                        Object.defineProperty(controller.prototype, inj.propertyName, { get: inj.getter });
                    });
                    typescript_class_helpers_1.CLASS.setSingletonObj(controller, new controller());
                    // Helpers.isBrowser && console.log(`[morphi] Singleton cleated for "${controller && controller.name}"`, CLASS.getSingleton(controller))
                })(currentCtrl);
            }
        });
        global_config_1.Global.vars.writeActiveRoutes();
    }
    //#endregion
    if (helpers_1.Helpers.isBrowser) {
        var notFound_1 = [];
        var providers = controllers.filter(function (ctrl) {
            var e = global_config_1.Global.vars.initFunc.find(function (e) { return ctrl === e.target; });
            if (e) {
                // console.log('current controller ', currentCtrl)
                e.initFN();
                return true;
            }
            else {
                var context = ctrl.prototype[symbols_1.SYMBOL.CLASS_DECORATOR_CONTEXT];
                if (!context) {
                    notFound_1.push(ctrl);
                    return false;
                }
                else {
                    context.initFN();
                    return true;
                }
            }
        });
        notFound_1.forEach(function (ctrl) {
            throw "Decorator \"@ENDPOINT(..)\" is missing on class " + typescript_class_helpers_1.CLASS.getName(ctrl);
        });
        providers.forEach(function (p) { return exports.Providers.push(p); });
    }
}
exports.init = init;
exports.Providers = [];
//# sourceMappingURL=init.js.map