"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var _ = require("lodash");
var io = require("socket.io-client");
var global_config_1 = require("../global-config");
var symbols_1 = require("../symbols");
var ng2_logger_1 = require("ng2-logger");
var typescript_class_helpers_1 = require("typescript-class-helpers");
var realtime_helper_1 = require("./realtime-helper");
var log = ng2_logger_1.Log.create('RealtimeBrowser', ng2_logger_1.Level.__NOTHING);
var RealtimeBrowser = /** @class */ (function () {
    function RealtimeBrowser() {
    }
    RealtimeBrowser.init = function () {
        if (RealtimeBrowser.connected) {
            log.warn('BROWSER ALREADY CONNECTED!');
            return;
        }
        RealtimeBrowser.connected = true;
        // RealtimeHelper.pathFor(SYMBOL.REALTIME.NAMESPACE).hostname
        var nspPath = {
            global: realtime_helper_1.RealtimeHelper.pathFor(),
            realtime: realtime_helper_1.RealtimeHelper.pathFor(symbols_1.SYMBOL.REALTIME.NAMESPACE)
        };
        log.i('NAMESPACE GLOBAL', nspPath.global.href);
        log.i('NAMESPACE REALTIME', nspPath.realtime.href);
        var global = io(nspPath.global.origin, {
            path: nspPath.global.pathname
        });
        global_config_1.Global.vars.socketNamespace.FE = global;
        global.on('connect', function () {
            log.i("conented to GLOBAL namespace " + global.nsp);
        });
        log.i('IT SHOULD CONNECT TO GLOBAL');
        var realtime = io(nspPath.realtime.origin, {
            path: nspPath.realtime.pathname
        });
        global_config_1.Global.vars.socketNamespace.FE_REALTIME = realtime;
        realtime.on('connect', function () {
            log.i("conented to REALTIME namespace " + realtime.nsp);
        });
        log.i('IT SHOULD CONNECT TO REALTIME');
    };
    RealtimeBrowser.TriggerChange = function (entity, property) {
        var _this = this;
        var constructFn = typescript_class_helpers_1.CLASS.getFromObject(entity);
        if (!constructFn) {
            log.er("Activate: Cannot retrive Class function from object", entity);
            return;
        }
        var className = typescript_class_helpers_1.CLASS.getName(constructFn);
        var id = entity.id;
        var unsub = function () {
            if (_.isString(property)) {
                _this.UnsubscribeEntityPropertyChanges(entity, property);
            }
            else {
                _this.UnsubscribeEntityChanges(entity);
            }
        };
        if (_.isString(property)) {
            var propertyInEntityKey = propertyInEntityKeyFn(entity, property);
            var arr = RealtimeBrowser.realtimeEntityPropertyListener[className][propertyInEntityKey];
            if (_.isArray(arr)) {
                arr.forEach(function (changeListenerFromArray) {
                    changeListenerFromArray(unsub);
                });
            }
        }
        else {
            var arr = RealtimeBrowser.realtimeEntityListener[className][entity.id];
            if (_.isArray(arr)) {
                arr.forEach(function (changeListenerFromArray) {
                    changeListenerFromArray(unsub);
                });
            }
        }
    };
    RealtimeBrowser.__SubscribeEntityChanges = function (entity, changesListener, property) {
        var _this = this;
        var id = entity.id;
        var propertyInEntityKey = propertyInEntityKeyFn(entity, property);
        if (!_.isNumber(id)) {
            console.error(entity);
            throw "[Morphi.Realtime.Browser.Subscribe] bad id = \"" + id + "\" for entity.";
        }
        var constructFn = typescript_class_helpers_1.CLASS.getFromObject(entity);
        if (!constructFn) {
            log.er("Activate: Cannot retrive Class function from object", entity);
            return;
        }
        var className = typescript_class_helpers_1.CLASS.getName(constructFn);
        this.checkObjects(className, entity, property, changesListener);
        var roomName = _.isString(property) ?
            symbols_1.SYMBOL.REALTIME.ROOM_NAME.UPDATE_ENTITY_PROPERTY(className, property, entity.id) :
            symbols_1.SYMBOL.REALTIME.ROOM_NAME.UPDATE_ENTITY(className, entity.id);
        var realtime = global_config_1.Global.vars.socketNamespace.FE_REALTIME;
        var ngZone = global_config_1.Global.vars.ngZone;
        // realtime.on('connect', () => {
        //   console.log(`conented to namespace ${realtime.nsp && realtime.nsp.name}`)
        if (_.isString(property)) {
            realtime.emit(symbols_1.SYMBOL.REALTIME.ROOM.SUBSCRIBE.ENTITY_PROPERTY_UPDATE_EVENTS, roomName);
            log.i('SUBSCRIBE TO ' + symbols_1.SYMBOL.REALTIME.EVENT.ENTITY_PROPTERY_UPDATE_BY_ID(className, property, entity.id));
        }
        else {
            realtime.emit(symbols_1.SYMBOL.REALTIME.ROOM.SUBSCRIBE.ENTITY_UPDATE_EVENTS, roomName);
            log.i('SUBSCRIBE TO ' + symbols_1.SYMBOL.REALTIME.EVENT.ENTITY_UPDATE_BY_ID(className, entity.id));
        }
        var callback = function (data) {
            var cb = _.debounce(function () {
                if (_.isFunction(changesListener)) {
                    var self_1 = _this;
                    var unsub_1 = function () {
                        if (_.isString(property)) {
                            _this.UnsubscribeEntityPropertyChanges(entity, property);
                        }
                        else {
                            _this.UnsubscribeEntityChanges(entity);
                        }
                    };
                    if (_.isString(property)) {
                        var arr = RealtimeBrowser.realtimeEntityPropertyListener[className][propertyInEntityKey];
                        if (_.isArray(arr)) {
                            arr.forEach(function (changeListenerFromArray) {
                                changeListenerFromArray(unsub_1);
                            });
                        }
                    }
                    else {
                        var arr = RealtimeBrowser.realtimeEntityListener[className][entity.id];
                        if (_.isArray(arr)) {
                            arr.forEach(function (changeListenerFromArray) {
                                changeListenerFromArray(unsub_1);
                            });
                        }
                    }
                }
                else {
                    log.er('Please define changedEntity');
                }
            }, 1000);
            log.i('data from socket without preparation (ngzone,rjxjs,transform)', data);
            if (ngZone) {
                ngZone.run(function () {
                    log.d('next from ngzone');
                    cb();
                });
            }
            else {
                log.d('next without ngzone');
                cb();
            }
            // if (ApplicationRef) {
            //   log.i('tick application ')
            //   ApplicationRef.tick()
            // }
        };
        var sub;
        if (_.isString(property)) {
            sub = realtime.on(symbols_1.SYMBOL.REALTIME.EVENT.ENTITY_PROPTERY_UPDATE_BY_ID(className, property, entity.id), callback);
            RealtimeBrowser.realtimeEntityPropertySockets[className][propertyInEntityKey] = sub;
            RealtimeBrowser.realtimeEntityPropertyListener[className][propertyInEntityKey].push(changesListener);
        }
        else {
            sub = realtime.on(symbols_1.SYMBOL.REALTIME.EVENT.ENTITY_UPDATE_BY_ID(className, entity.id), callback);
            RealtimeBrowser.realtimeEntitySockets[className][entity.id] = sub;
            RealtimeBrowser.realtimeEntityListener[className][entity.id].push(changesListener);
        }
    };
    RealtimeBrowser.addDupicateRealtimeEntityListener = function (entity, changesListener, property) {
        var className = typescript_class_helpers_1.CLASS.getNameFromObject(entity);
        if (_.isUndefined(RealtimeBrowser.realtimeEntityPropertyListener[className])) {
            RealtimeBrowser.realtimeEntityPropertyListener[className] = {};
        }
        if (_.isString(property)) {
            var propertyInEntityKey = propertyInEntityKeyFn(entity, property);
            RealtimeBrowser.realtimeEntityPropertyListener[className][propertyInEntityKey].push(changesListener);
        }
        else {
            RealtimeBrowser.realtimeEntityListener[className][entity.id].push(changesListener);
        }
    };
    RealtimeBrowser.SubscribeEntityChanges = function (entity, changesListener) {
        return RealtimeBrowser.__SubscribeEntityChanges(entity, changesListener);
    };
    RealtimeBrowser.SubscribeEntityPropertyChanges = function (entity, property, changesListener) {
        return RealtimeBrowser.__SubscribeEntityChanges(entity, changesListener, property);
    };
    RealtimeBrowser.checkObjects = function (className, entity, property, changesListener) {
        if (_.isString(property)) {
            if (_.isUndefined(RealtimeBrowser.realtimeEntityPropertySockets[className])) {
                RealtimeBrowser.realtimeEntityPropertySockets[className] = {};
            }
            if (_.isUndefined(RealtimeBrowser.realtimeEntityPropertyListener[className])) {
                RealtimeBrowser.realtimeEntityPropertyListener[className] = {};
            }
            var propertyInEntityKey = propertyInEntityKeyFn(entity, property);
            if (!_.isArray(RealtimeBrowser.realtimeEntityPropertyListener[className][propertyInEntityKey])) {
                RealtimeBrowser.realtimeEntityPropertyListener[className][propertyInEntityKey] = [];
            }
            if (_.isObject(RealtimeBrowser.realtimeEntityPropertyListener[className][propertyInEntityKey])) {
                log.w("alread listen to this object property: " + property + " realtime events", entity);
                if (!RealtimeBrowser.realtimeEntityPropertyListener[className][propertyInEntityKey].includes(changesListener)) {
                    log.d("new change listener added, property: " + property, entity);
                    RealtimeBrowser.realtimeEntityPropertyListener[className][propertyInEntityKey].push(changesListener);
                }
                else {
                    log.d("change listener already exist, property: " + property, entity);
                }
                return false;
            }
        }
        else {
            if (_.isUndefined(RealtimeBrowser.realtimeEntitySockets[className])) {
                RealtimeBrowser.realtimeEntitySockets[className] = {};
            }
            if (_.isUndefined(RealtimeBrowser.realtimeEntityListener[className])) {
                RealtimeBrowser.realtimeEntityListener[className] = {};
            }
            if (!_.isArray(RealtimeBrowser.realtimeEntityListener[className][entity.id])) {
                RealtimeBrowser.realtimeEntityListener[className][entity.id] = [];
            }
            if (_.isObject(RealtimeBrowser.realtimeEntitySockets[className][entity.id])) {
                log.w('alread listen to this object realtime events', entity);
                if (!RealtimeBrowser.realtimeEntityListener[className][entity.id].includes(changesListener)) {
                    log.d('new change listener added', entity);
                    RealtimeBrowser.realtimeEntityListener[className][entity.id].push(changesListener);
                }
                else {
                    log.d('change listener already exist', entity);
                }
                return false;
            }
        }
        return true;
    };
    RealtimeBrowser.__UnsubscribeEntityChanges = function (entity, property, includePropertyChanges, classFN) {
        var _this = this;
        if (includePropertyChanges === void 0) { includePropertyChanges = false; }
        if (includePropertyChanges) {
            typescript_class_helpers_1.CLASS.describeProperites(typescript_class_helpers_1.CLASS.getFromObject(entity)).forEach(function (property) {
                _this.UnsubscribeEntityPropertyChanges(entity, property);
            });
        }
        var id = entity.id;
        if (!_.isNumber(id)) {
            console.error(entity);
            throw "[Morphi.Realtime.Browser.Unsubscribe] bad id = \"" + id + "\" for entity.";
        }
        var constructFn = _.isFunction(classFN) ? classFN : typescript_class_helpers_1.CLASS.getFromObject(entity);
        if (!constructFn) {
            log.er("Deactivate: Cannot retrive Class function from object", entity);
            return;
        }
        var className = typescript_class_helpers_1.CLASS.getName(constructFn);
        var realtime = global_config_1.Global.vars.socketNamespace.FE_REALTIME;
        var roomName = _.isString(property) ?
            symbols_1.SYMBOL.REALTIME.ROOM_NAME.UPDATE_ENTITY_PROPERTY(className, property, entity.id) :
            symbols_1.SYMBOL.REALTIME.ROOM_NAME.UPDATE_ENTITY(className, entity.id);
        var sub;
        if (_.isString(property)) {
            var propertyInEntityKey = propertyInEntityKeyFn(entity, property);
            sub = RealtimeBrowser.realtimeEntityPropertySockets[className] && RealtimeBrowser.realtimeEntityPropertySockets[className][propertyInEntityKey];
        }
        else {
            sub = RealtimeBrowser.realtimeEntitySockets[className] && RealtimeBrowser.realtimeEntitySockets[className][entity.id];
        }
        if (sub) {
            sub.removeAllListeners();
            if (_.isString(property)) {
                delete RealtimeBrowser.realtimeEntityPropertyListener[className][propertyInEntityKey];
            }
            else {
                delete RealtimeBrowser.realtimeEntityListener[className][entity.id];
            }
            console.info("Unsubscribe OK from entit: " + className + " - " + (propertyInEntityKey ? propertyInEntityKey : entity.id));
        }
        else {
            console.info("Unsubscribe not found from entity: " + className + " - " + (propertyInEntityKey ? propertyInEntityKey : entity.id));
        }
        if (_.isString(property)) {
            realtime.emit(symbols_1.SYMBOL.REALTIME.ROOM.UNSUBSCRIBE.ENTITY_PROPERTY_UPDATE_EVENTS, roomName);
        }
        else {
            realtime.emit(symbols_1.SYMBOL.REALTIME.ROOM.UNSUBSCRIBE.ENTITY_UPDATE_EVENTS, roomName);
        }
    };
    RealtimeBrowser.UnsubscribeEverything = function () {
        var _this = this;
        Object.keys(this.realtimeEntitySockets).forEach(function (className) {
            Object.keys(_this.realtimeEntitySockets[className]).forEach(function (entityId) {
                _this.__UnsubscribeEntityChanges({ id: entityId }, undefined, true, typescript_class_helpers_1.CLASS.getBy(className));
            });
        });
    };
    RealtimeBrowser.UnsubscribeEntityChanges = function (entity, includePropertyChanges) {
        if (includePropertyChanges === void 0) { includePropertyChanges = false; }
        return RealtimeBrowser.__UnsubscribeEntityChanges(entity, undefined, includePropertyChanges);
    };
    RealtimeBrowser.UnsubscribeEntityPropertyChanges = function (entity, property) {
        return RealtimeBrowser.__UnsubscribeEntityChanges(entity, property);
    };
    RealtimeBrowser.connected = false;
    RealtimeBrowser.realtimeEntityListener = {};
    RealtimeBrowser.realtimeEntityPropertyListener = {};
    RealtimeBrowser.realtimeEntitySockets = {};
    RealtimeBrowser.realtimeEntityPropertySockets = {};
    return RealtimeBrowser;
}());
exports.RealtimeBrowser = RealtimeBrowser;
function propertyInEntityKeyFn(entity, property) {
    return "" + entity.id + property;
}
//# sourceMappingURL=realtime-browser.js.map