"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var global_config_1 = require("../global-config");
var _ = require("lodash");
var io = require("socket.io");
var symbols_1 = require("../symbols");
//#endregion
var ng2_logger_1 = require("ng2-logger");
var realtime_helper_1 = require("./realtime-helper");
var typescript_class_helpers_1 = require("typescript-class-helpers");
var log = ng2_logger_1.Log.create('RealtimeNodejs', ng2_logger_1.Level.__NOTHING);
var RealtimeNodejs = /** @class */ (function () {
    function RealtimeNodejs() {
    }
    //#region @backend
    RealtimeNodejs.init = function (http) {
        var nspPath = {
            global: realtime_helper_1.RealtimeHelper.pathFor(),
            realtime: realtime_helper_1.RealtimeHelper.pathFor(symbols_1.SYMBOL.REALTIME.NAMESPACE)
        };
        global_config_1.Global.vars.socketNamespace.BE = io(http, {
            path: nspPath.global.pathname
        });
        var ioGlobalNsp = global_config_1.Global.vars.socketNamespace.BE;
        ioGlobalNsp.on('connection', function (clientSocket) {
            log.i('client conected to namespace', clientSocket.nsp.name);
        });
        log.i("CREATE GLOBAL NAMESPACE: '" + ioGlobalNsp.path() + "' , path: '" + nspPath.global.pathname + "'");
        var ioRealtimeNsp = io(http, {
            path: nspPath.realtime.pathname
        });
        log.i("CREATE REALTIME NAMESPACE: '" + ioRealtimeNsp.path() + "' , path: '" + nspPath.realtime.pathname + "' ");
        global_config_1.Global.vars.socketNamespace.BE_REALTIME = ioRealtimeNsp;
        ioRealtimeNsp.on('connection', function (clientSocket) {
            log.i('client conected to namespace', clientSocket.nsp.name);
            clientSocket.on(symbols_1.SYMBOL.REALTIME.ROOM.SUBSCRIBE.ENTITY_UPDATE_EVENTS, function (room) {
                log.i("Joining room " + room + " in namespace  REALTIME");
                clientSocket.join(room);
            });
            clientSocket.on(symbols_1.SYMBOL.REALTIME.ROOM.SUBSCRIBE.ENTITY_PROPERTY_UPDATE_EVENTS, function (room) {
                log.i("Joining room " + room + " in namespace REALTIME ");
                clientSocket.join(room);
            });
            clientSocket.on(symbols_1.SYMBOL.REALTIME.ROOM.UNSUBSCRIBE.ENTITY_UPDATE_EVENTS, function (room) {
                log.i("Leaving room " + room + " in namespace REALTIME ");
                clientSocket.leave(room);
            });
            clientSocket.on(symbols_1.SYMBOL.REALTIME.ROOM.UNSUBSCRIBE.ENTITY_PROPERTY_UPDATE_EVENTS, function (room) {
                log.i("Leaving room " + room + " in namespace REALTIME ");
                clientSocket.leave(room);
            });
        });
    };
    RealtimeNodejs.__TrigggerEntityChanges = function (entity, property) {
        var keyPropertyName = 'id';
        if (!entity || !entity[keyPropertyName]) {
            console.error("Entity without iD !!!! ", entity);
            return;
        }
        var id = entity[keyPropertyName];
        // Global.vars.socket.BE.sockets.in()\
        var constructFn = typescript_class_helpers_1.CLASS.getFromObject(entity);
        // console.log('construcFN', constructFn)
        if (!constructFn) {
            log.d('not found class function from', entity);
        }
        else {
            var className_1 = typescript_class_helpers_1.CLASS.getName(constructFn);
            var modelSocketRoomPath_1 = _.isString(property) ?
                symbols_1.SYMBOL.REALTIME.ROOM_NAME.UPDATE_ENTITY_PROPERTY(className_1, property, entity.id) :
                symbols_1.SYMBOL.REALTIME.ROOM_NAME.UPDATE_ENTITY(className_1, id);
            // console.log(`Push entity${_.isString(property) ? ('.' + property) : ''} to room with path: ${modelSocketRoomPath}`)
            var eventName_1 = _.isString(property) ?
                symbols_1.SYMBOL.REALTIME.EVENT.ENTITY_PROPTERY_UPDATE_BY_ID(className_1, property, id) :
                symbols_1.SYMBOL.REALTIME.EVENT.ENTITY_UPDATE_BY_ID(className_1, id);
            var job_1 = function () {
                if (_.isString(property)) {
                    // console.log('populate entity property change to ', SYMBOL.REALTIME.EVENT.ENTITY_UPDATE_BY_ID(className, id))
                    global_config_1.Global.vars.socketNamespace.BE_REALTIME.in(modelSocketRoomPath_1)
                        .emit(eventName_1, '');
                }
                else {
                    log.i('populate entity change to ', symbols_1.SYMBOL.REALTIME.EVENT.ENTITY_UPDATE_BY_ID(className_1, id));
                    global_config_1.Global.vars.socketNamespace.BE_REALTIME.in(modelSocketRoomPath_1)
                        .emit(eventName_1, '');
                }
            };
            if (!_.isFunction(RealtimeNodejs.jobs[eventName_1])) {
                log.i('CREATE FUNCTION DEBOUNCE');
                RealtimeNodejs.jobs[eventName_1] = _.debounce(function () {
                    job_1();
                }, 500);
            }
            RealtimeNodejs.jobs[eventName_1]();
        }
    };
    RealtimeNodejs.TrigggerEntityPropertyChanges = function (entity, property) {
        if (_.isArray(property)) {
            property.forEach(function (p) {
                RealtimeNodejs.__TrigggerEntityChanges(entity, p);
            });
            return;
        }
        RealtimeNodejs.__TrigggerEntityChanges(entity, property);
    };
    RealtimeNodejs.TrigggerEntityChanges = function (entity) {
        RealtimeNodejs.__TrigggerEntityChanges(entity);
    };
    RealtimeNodejs.jobs = {};
    return RealtimeNodejs;
}());
exports.RealtimeNodejs = RealtimeNodejs;
//# sourceMappingURL=realtime-nodejs.js.map